@@ -6,20 +6,12 @@ use My::Platform;
 sub skip_combinations {
   my @combinations;
 
-  # disable innodb/xtradb combinatons for configurations that were not built
+  # disable innodb combinations for configurations that were not built
   push @combinations, 'innodb_plugin' unless $ENV{HA_INNODB_SO};
 
-  push @combinations, qw(xtradb innodb) unless $::mysqld_variables{'innodb'} eq "ON";
+  push @combinations, 'innodb' unless $::mysqld_variables{'innodb'} eq "ON";
 
-  # unconditionally, for now in 10.2. Later it could check for xtradb I_S plugins
-  push @combinations, 'xtradb';
-
-  # XtraDB is RECOMPILE_FOR_EMBEDDED, ha_xtradb.so cannot work with embedded server
-  push @combinations, 'xtradb_plugin' if not $ENV{HA_XTRADB_SO}
-                                          or $::opt_embedded_server;
-
-  my %skip = ( 'include/have_innodb.combinations' => [ @combinations ],
-               'include/have_xtradb.combinations' => [ @combinations ]);
+  my %skip = ( 'include/have_innodb.combinations' => [ @combinations ]);
 
   $skip{'include/innodb_encrypt_log.combinations'} = [ 'crypt' ]
                 unless $ENV{DEBUG_KEY_MANAGEMENT_SO};
@@ -56,7 +48,7 @@ sub skip_combinations {
 
   $skip{'include/not_windows.inc'} = 'Requires not Windows' if IS_WINDOWS;
 
-  $skip{'t/plugin_loaderr.test'} = 'needs compiled-in innodb'
+  $skip{'main/plugin_loaderr.test'} = 'needs compiled-in innodb'
             unless $::mysqld_variables{'innodb'} eq "ON";
 
   $skip{'include/have_mariabackup.inc'} = 'Need mariabackup'
@@ -68,12 +60,6 @@ sub skip_combinations {
   $skip{'include/have_mariabackup.inc'} = 'Need socat or nc'
             unless IS_WINDOWS || $ENV{MTR_GALERA_TFMT};
 
-  $skip{'include/have_xtrabackup.inc'} = 'Need innobackupex'
-            unless IS_WINDOWS || ::which("innobackupex");
-
-  $skip{'include/have_xtrabackup.inc'} = 'Need socat or nc'
-            unless IS_WINDOWS || $ENV{MTR_GALERA_TFMT};
-
   $skip{'include/have_garbd.inc'} = 'Need garbd'
             unless ::have_garbd();
 
@@ -91,15 +77,15 @@ sub skip_combinations {
   }
   $skip{'include/check_ipv6.inc'} = 'No IPv6' unless ipv6_ok();
 
-  $skip{'t/openssl_6975.test'} = 'no or wrong openssl version'
+  $skip{'main/openssl_6975.test'} = 'no or wrong openssl version'
     unless $::mysqld_variables{'version-ssl-library'} =~ /OpenSSL (\S+)/
        and $1 ge "1.0.1d" and $1 lt "1.1.1";
 
-  $skip{'t/ssl_7937.combinations'} = [ 'x509v3' ]
+  $skip{'main/ssl_7937.combinations'} = [ 'x509v3' ]
     unless $::mysqld_variables{'version-ssl-library'} =~ /OpenSSL (\S+)/
        and $1 ge "1.0.2";
 
-  $skip{'t/ssl_verify_ip.test'} = 'x509v3 support required'
+  $skip{'main/ssl_verify_ip.test'} = 'x509v3 support required'
     unless $::mysqld_variables{'version-ssl-library'} =~ /OpenSSL (\S+)/
        and $1 ge "1.0.2";
 
