@@ -319,13 +319,6 @@ enum que_thr_lock_t {
 	QUE_THR_LOCK_TABLE
 };
 
-/** From where the cursor position is counted */
-enum que_cur_t {
-	QUE_CUR_NOT_DEFINED,
-	QUE_CUR_START,
-	QUE_CUR_END
-};
-
 /* Query graph query thread node: the fields are protected by the
 trx_t::mutex with the exceptions named below */
 
@@ -402,18 +395,7 @@ struct que_fork_t{
 					generated by the parser, or NULL
 					if the graph was created 'by hand' */
 	pars_info_t*	info;		/*!< info struct, or NULL */
-	/* The following cur_... fields are relevant only in a select graph */
 
-	ulint		cur_end;	/*!< QUE_CUR_NOT_DEFINED, QUE_CUR_START,
-					QUE_CUR_END */
-	ulint		cur_pos;	/*!< if there are n rows in the result
-					set, values 0 and n + 1 mean before
-					first row, or after last row, depending
-					on cur_end; values 1...n mean a row
-					index */
-	ibool		cur_on_row;	/*!< TRUE if cursor is on a row, i.e.,
-					it is not before the first row or
-					after the last row */
 	sel_node_t*	last_sel_node;	/*!< last executed select node, or NULL
 					if none */
 	UT_LIST_NODE_T(que_fork_t)
