@@ -716,10 +716,9 @@ int maria_create(const char *name, enum data_file_type datafile_type,
     share.base.extra_options|= MA_EXTRA_OPTIONS_INSERT_ORDER;
   }
 
-  share.state.state.key_file_length= MY_ALIGN(info_length, maria_block_size);
   DBUG_PRINT("info", ("info_length: %u", info_length));
   /* There are only 16 bits for the total header length. */
-  if (share.state.state.key_file_length > 65535)
+  if (info_length > 65535)
   {
     my_printf_error(HA_WRONG_CREATE_OPTION,
                     "Aria table '%s' has too many columns and/or "
@@ -776,7 +775,8 @@ int maria_create(const char *name, enum data_file_type datafile_type,
 
   maria_set_all_keys_active(share.state.key_map, keys);
 
-  share.base.keystart = share.state.state.key_file_length;
+  share.base.keystart = share.state.state.key_file_length=
+    MY_ALIGN(info_length, maria_block_size);
   share.base.max_key_block_length= maria_block_size;
   share.base.max_key_length=ALIGN_SIZE(max_key_length+4);
   share.base.records=ci->max_rows;
@@ -1153,7 +1153,7 @@ int maria_create(const char *name, enum data_file_type datafile_type,
     DBUG_EXECUTE_IF("maria_crash_create_table",
                     {
                       DBUG_PRINT("maria_crash_create_table", ("now"));
-                      DBUG_ABORT();
+                      DBUG_SUICIDE();
                     });
     /*
       store LSN into file, needed for Recovery to not be confused if a
@@ -1163,6 +1163,7 @@ int maria_create(const char *name, enum data_file_type datafile_type,
                                   FALSE, TRUE))
       goto err;
     my_free(log_data);
+    log_data= 0;
   }
 
   if (!(flags & HA_DONT_TOUCH_DATA))
@@ -1431,7 +1432,7 @@ int _ma_update_state_lsns(MARIA_SHARE *share, LSN lsn, TrID create_trid,
      @retval 1      error (disk problem)
 */
 
-#if (_MSC_VER == 1310)
+#if defined(_MSC_VER) && (_MSC_VER == 1310)
 /*
  Visual Studio 2003 compiler produces internal compiler error
  in this function. Disable optimizations to workaround.
@@ -1504,6 +1505,6 @@ int _ma_update_state_lsns_sub(MARIA_SHARE *share, LSN lsn, TrID create_trid,
                     MARIA_FILE_CREATE_TRID_OFFSET, MYF(MY_NABP)) ||
           (do_sync && mysql_file_sync(file, MYF(0))));
 }
-#if (_MSC_VER == 1310)
+#if defined(_MSC_VER) && (_MSC_VER == 1310)
 #pragma optimize("",on)
 #endif /*VS2003 compiler bug workaround*/
